@@ -7,6 +7,7 @@
 #include <rem.h>
 #include <windows.h>
 #include <mmsystem.h>
+#include <mmreg.h>
 #include <baresip.h>
 #include "winwave.h"
 
@@ -22,6 +23,7 @@ struct ausrc_st {
 	HWAVEIN wavein;
 	volatile bool rdy;
 	size_t inuse;
+	size_t sampsz;
 	ausrc_read_h *rh;
 	void *arg;
 };
@@ -104,7 +106,8 @@ static void CALLBACK waveInCallback(HWAVEOUT hwo,
 		if (st->inuse < (READ_BUFFERS-1))
 			add_wave_in(st);
 
-		st->rh((void *)wh->lpData, wh->dwBytesRecorded/2, st->arg);
+		st->rh((void *)wh->lpData, wh->dwBytesRecorded/st->sampsz,
+		       st->arg);
 
 		waveInUnprepareHeader(st->wavein, wh, sizeof(*wh));
 		st->inuse--;
@@ -116,36 +119,24 @@ static void CALLBACK waveInCallback(HWAVEOUT hwo,
 }
 
 
-static unsigned int find_dev(const char *name)
-{
-	WAVEINCAPS wic;
-	unsigned int i, nInDevices = waveInGetNumDevs();
-
-	if (!str_isset(name))
-		return WAVE_MAPPER;
-
-	for (i=0; i<nInDevices; i++) {
-		if (waveInGetDevCaps(i, &wic,
-				     sizeof(WAVEINCAPS))==MMSYSERR_NOERROR) {
-
-			if (0 == str_casecmp(name, wic.szPname)) {
-				return i;
-			}
-		}
-	}
-
-	return WAVE_MAPPER;
-}
-
-
 static int read_stream_open(struct ausrc_st *st, const struct ausrc_prm *prm,
 			    unsigned int dev)
 {
 	WAVEFORMATEX wfmt;
 	MMRESULT res;
 	uint32_t sampc;
+	unsigned format;
 	int i, err = 0;
 
+	st->sampsz = aufmt_sample_size(prm->fmt);
+
+	format = winwave_get_format(prm->fmt);
+	if (format == WAVE_FORMAT_UNKNOWN) {
+		warning("winwave: source: unsupported sample format (%s)\n",
+			aufmt_name(prm->fmt));
+		return ENOTSUP;
+	}
+
 	/* Open an audio INPUT stream. */
 	st->wavein = NULL;
 	st->pos = 0;
@@ -155,15 +146,15 @@ static int read_stream_open(struct ausrc_st *st, const struct ausrc_prm *prm,
 
 	for (i = 0; i < READ_BUFFERS; i++) {
 		memset(&st->bufs[i].wh, 0, sizeof(WAVEHDR));
-		st->bufs[i].mb = mbuf_alloc(2 * sampc);
+		st->bufs[i].mb = mbuf_alloc(st->sampsz * sampc);
 		if (!st->bufs[i].mb)
 			return ENOMEM;
 	}
 
-	wfmt.wFormatTag      = WAVE_FORMAT_PCM;
+	wfmt.wFormatTag      = format;
 	wfmt.nChannels       = prm->ch;
 	wfmt.nSamplesPerSec  = prm->srate;
-	wfmt.wBitsPerSample  = 16;
+	wfmt.wBitsPerSample  = st->sampsz * 8;
 	wfmt.nBlockAlign     = (prm->ch * wfmt.wBitsPerSample) / 8;
 	wfmt.nAvgBytesPerSec = wfmt.nSamplesPerSec * wfmt.nBlockAlign;
 	wfmt.cbSize          = 0;
@@ -173,7 +164,7 @@ static int read_stream_open(struct ausrc_st *st, const struct ausrc_prm *prm,
 			  (DWORD_PTR) st,
 			  CALLBACK_FUNCTION | WAVE_FORMAT_DIRECT);
 	if (res != MMSYSERR_NOERROR) {
-		warning("winwave: waveInOpen: failed %d\n", err);
+		warning("winwave: waveInOpen: failed res=%d\n", res);
 		return EINVAL;
 	}
 
@@ -187,6 +178,36 @@ static int read_stream_open(struct ausrc_st *st, const struct ausrc_prm *prm,
 }
 
 
+static int winwave_get_dev_name(unsigned int i, char name[32])
+{
+	WAVEINCAPS wic;
+	int err = 0;
+
+	if (waveInGetDevCaps(i, &wic,
+			     sizeof(WAVEINCAPS)) == MMSYSERR_NOERROR) {
+		str_ncpy(name, wic.szPname, 32);
+	}
+	else {
+		err = ENODEV;
+	}
+
+	return err;
+}
+
+
+static unsigned int winwave_get_num_devs(void)
+{
+	return waveInGetNumDevs();
+}
+
+
+static int find_dev(const char *name, unsigned int *dev)
+{
+	return winwave_enum_devices(name, NULL, dev, winwave_get_num_devs,
+				    winwave_get_dev_name);
+}
+
+
 int winwave_src_alloc(struct ausrc_st **stp, const struct ausrc *as,
 		      struct media_ctx **ctx,
 		      struct ausrc_prm *prm, const char *device,
@@ -194,6 +215,7 @@ int winwave_src_alloc(struct ausrc_st **stp, const struct ausrc *as,
 {
 	struct ausrc_st *st;
 	int err;
+	unsigned int dev;
 
 	(void)ctx;
 	(void)errh;
@@ -201,10 +223,9 @@ int winwave_src_alloc(struct ausrc_st **stp, const struct ausrc *as,
 	if (!stp || !as || !prm)
 		return EINVAL;
 
-	if (prm->fmt != AUFMT_S16LE) {
-		warning("winwave: source: unsupported sample format (%s)\n",
-			aufmt_name(prm->fmt));
-		return ENOTSUP;
+	err = find_dev(device, &dev);
+	if (err) {
+		return err;
 	}
 
 	st = mem_zalloc(sizeof(*st), ausrc_destructor);
@@ -215,7 +236,7 @@ int winwave_src_alloc(struct ausrc_st **stp, const struct ausrc *as,
 	st->rh  = rh;
 	st->arg = arg;
 
-	err = read_stream_open(st, prm, find_dev(device));
+	err |= read_stream_open(st, prm, dev);
 
 	if (err)
 		mem_deref(st);
@@ -224,3 +245,23 @@ int winwave_src_alloc(struct ausrc_st **stp, const struct ausrc *as,
 
 	return err;
 }
+
+
+static int set_available_devices(struct list *dev_list)
+{
+	return winwave_enum_devices(NULL, dev_list, NULL,
+				    winwave_get_num_devs,
+				    winwave_get_dev_name);
+}
+
+
+int winwave_src_init(struct ausrc *as)
+{
+	if (!as) {
+		return EINVAL;
+	}
+
+	list_init(&as->dev_list);
+
+	return set_available_devices(&as->dev_list);
+}
