@@ -264,6 +264,8 @@ static void audio_destructor(void *arg)
 {
 	struct audio *a = arg;
 
+	debug("audio: destroyed (started=%d)\n", a->started);
+
 	stop_tx(&a->tx, a);
 	stop_rx(&a->rx);
 
@@ -324,14 +326,14 @@ static inline uint32_t get_srate(const struct aucodec *ac)
 
 
 /*
- * Get the DSP channels for an audio-codec (exception for MPA)
+ * Get the DSP channels for an audio-codec
  */
 static inline uint32_t get_ch(const struct aucodec *ac)
 {
 	if (!ac)
 		return 0;
 
-	return !str_casecmp(ac->name, "MPA") ? 2 : ac->ch;
+	return ac->ch;
 }
 
 
@@ -354,28 +356,21 @@ static bool aucodec_equal(const struct aucodec *a, const struct aucodec *b)
 }
 
 
-static int add_audio_codec(struct audio *a, struct sdp_media *m,
-			   struct aucodec *ac)
+static int add_audio_codec(struct sdp_media *m, struct aucodec *ac)
 {
-	if (!in_range(&a->cfg.srate, get_srate(ac))) {
-		debug("audio: skip %uHz codec (audio range %uHz - %uHz)\n",
-		      get_srate(ac), a->cfg.srate.min, a->cfg.srate.max);
-		return 0;
-	}
-
-	if (!in_range(&a->cfg.channels, get_ch(ac))) {
-		debug("audio: skip codec with %uch (audio range %uch-%uch)\n",
-		      get_ch(ac), a->cfg.channels.min, a->cfg.channels.max);
-		return 0;
-	}
-
 	if (ac->crate < 8000) {
 		warning("audio: illegal clock rate %u\n", ac->crate);
 		return EINVAL;
 	}
 
+	if (ac->ch == 0 || ac->pch == 0) {
+		warning("audio: illegal channels for audio codec '%s'\n",
+			ac->name);
+		return EINVAL;
+	}
+
 	return sdp_format_add(NULL, m, false, ac->pt, ac->name, ac->crate,
-			      ac->ch, ac->fmtp_ench, ac->fmtp_cmph, ac, false,
+			      ac->pch, ac->fmtp_ench, ac->fmtp_cmph, ac, false,
 			      "%s", ac->fmtp);
 }
 
@@ -420,6 +415,7 @@ static void encode_rtp_send(struct audio *a, struct autx *tx,
 	size_t sampc_rtp;
 	size_t len;
 	size_t ext_len = 0;
+	uint32_t ts_delta = 0;
 	int err;
 
 	if (!tx->ac || !tx->ac->ench)
@@ -455,8 +451,12 @@ static void encode_rtp_send(struct audio *a, struct autx *tx,
 			   tx->enc_fmt, sampv, sampc);
 
 	if ((err & 0xffff0000) == 0x00010000) {
+
 		/* MPA needs some special treatment here */
-		tx->ts_ext = err & 0xffff;
+
+		ts_delta = err & 0xffff;
+		sampc = 0;
+
 		err = 0;
 	}
 	else if (err) {
@@ -478,6 +478,11 @@ static void encode_rtp_send(struct audio *a, struct autx *tx,
 			if (err)
 				goto out;
 		}
+
+		if (ts_delta) {
+			tx->ts_ext += ts_delta;
+			goto out;
+		}
 	}
 
 	/* Convert from audio samplerate to RTP clockrate */
@@ -488,7 +493,7 @@ static void encode_rtp_send(struct audio *a, struct autx *tx,
 	 * However, MPA support variable packet durations. Thus, MPA
 	 * should update the ts according to its current internal state.
 	 */
-	frame_size = sampc_rtp / get_ch(tx->ac);
+	frame_size = sampc_rtp / tx->ac->pch;
 
 	tx->ts_ext += (uint32_t)frame_size;
 
@@ -573,23 +578,15 @@ static void poll_aubuf_tx(struct audio *a)
 		sampc = sampc_rs;
 	}
 
-	if (tx->enc_fmt == AUFMT_S16LE) {
+	/* Process exactly one audio-frame in list order */
+	for (le = tx->filtl.head; le; le = le->next) {
+		struct aufilt_enc_st *st = le->data;
 
-		/* Process exactly one audio-frame in list order */
-		for (le = tx->filtl.head; le; le = le->next) {
-			struct aufilt_enc_st *st = le->data;
-
-			if (st->af && st->af->ench)
-				err |= st->af->ench(st, sampv, &sampc);
-		}
-		if (err) {
-			warning("audio: aufilter encode: %m\n", err);
-		}
+		if (st->af && st->af->ench)
+			err |= st->af->ench(st, sampv, &sampc);
 	}
-	else if (!list_isempty(&tx->filtl)) {
-		warning("audio: skipping audio-filters due to"
-			" incompatible format (%s)\n",
-			aufmt_name(tx->enc_fmt));
+	if (err) {
+		warning("audio: aufilter encode: %m\n", err);
 	}
 
 	/* Encode and send */
@@ -718,7 +715,7 @@ static void ausrc_error_handler(int err, const char *str, void *arg)
 }
 
 
-static int pt_handler(struct audio *a, uint8_t pt_old, uint8_t pt_new)
+static int update_payload_type(struct audio *a, uint8_t pt_old, uint8_t pt_new)
 {
 	const struct sdp_format *lc;
 
@@ -785,19 +782,12 @@ static int aurx_stream_decode(struct aurx *rx, struct mbuf *mb)
 		goto out;
 	}
 
-	if (rx->dec_fmt == AUFMT_S16LE) {
-		/* Process exactly one audio-frame in reverse list order */
-		for (le = rx->filtl.tail; le; le = le->prev) {
-			struct aufilt_dec_st *st = le->data;
+	/* Process exactly one audio-frame in reverse list order */
+	for (le = rx->filtl.tail; le; le = le->prev) {
+		struct aufilt_dec_st *st = le->data;
 
-			if (st->af && st->af->dech)
-				err |= st->af->dech(st, rx->sampv, &sampc);
-		}
-	}
-	else if (!list_isempty(&rx->filtl)) {
-		warning("audio: skipping audio-filters due to"
-			" incompatible format (%s)\n",
-			aufmt_name(rx->dec_fmt));
+		if (st->af && st->af->dech)
+			err |= st->af->dech(st, rx->sampv, &sampc);
 	}
 
 	if (!rx->aubuf)
@@ -830,8 +820,10 @@ static int aurx_stream_decode(struct aurx *rx, struct mbuf *mb)
 
 		++rx->stats.aubuf_overrun;
 
+#if 0
 		debug("audio: rx aubuf overrun (total %llu)\n",
 		      rx->stats.aubuf_overrun);
+#endif
 	}
 
 	if (rx->play_fmt == rx->dec_fmt) {
@@ -894,6 +886,8 @@ static void stream_recv_handler(const struct rtp_header *hdr,
 	int wrap;
 	int err;
 
+	MAGIC_CHECK(a);
+
 	if (!mb)
 		goto out;
 
@@ -917,7 +911,7 @@ static void stream_recv_handler(const struct rtp_header *hdr,
 	/* XXX: this logic should be moved to stream.c */
 	if (hdr->pt != rx->pt) {
 
-		err = pt_handler(a, rx->pt, hdr->pt);
+		err = update_payload_type(a, rx->pt, hdr->pt);
 		if (err)
 			return;
 	}
@@ -942,10 +936,10 @@ static void stream_recv_handler(const struct rtp_header *hdr,
 
 		uint64_t ext_last, ext_now;
 
-		ext_last = calc_extended_timestamp(rx->ts_recv.num_wraps,
+		ext_last = timestamp_calc_extended(rx->ts_recv.num_wraps,
 						   rx->ts_recv.last);
 
-		ext_now = calc_extended_timestamp(rx->ts_recv.num_wraps,
+		ext_now = timestamp_calc_extended(rx->ts_recv.num_wraps,
 						  hdr->ts);
 
 		if (ext_now <= ext_last) {
@@ -962,9 +956,7 @@ static void stream_recv_handler(const struct rtp_header *hdr,
 		}
 	}
 	else {
-		rx->ts_recv.first = hdr->ts;
-		rx->ts_recv.last = hdr->ts;
-		rx->ts_recv.is_set = true;
+		timestamp_set(&rx->ts_recv, hdr->ts);
 	}
 
 	wrap = timestamp_wrap(hdr->ts, rx->ts_recv.last);
@@ -1078,6 +1070,55 @@ static int set_ebuacip_params(struct audio *au, uint32_t ptime)
 }
 
 
+static bool ebuacip_handler(const char *name, const char *value, void *arg)
+{
+	struct sdp_media *sdp;
+	struct audio *au = arg;
+	struct aurx *rx = &au->rx;
+	struct pl type, val;
+	uint32_t frames;
+	(void)name;
+
+	if (0 == re_regex(value, str_len(value),
+		"jbdef [0-9]+ [^ ]+ [0-9]+",
+		NULL, &type, &val)) {
+
+		const uint32_t ptime = rx->ptime ? rx->ptime : 20;
+
+		frames = pl_u32(&val) / ptime;
+
+		if (0 == pl_strcasecmp(&type,"fixed")) {
+
+			uint32_t frames_min;
+
+			/*
+			fixed jb, set to frames -1 as min and frames as max.
+			*/
+
+			if (frames > 1)
+				frames_min = frames - 1;
+			else
+				frames_min = 1;
+
+			stream_jbuf_reset(au->strm, frames_min, frames);
+		}
+		else if (0 == pl_strcasecmp(&type, "auto")) {
+			/*
+			at the moment only min value is known,
+			therefor max value is here set to 2 times min value
+			This needs to be addressed later
+			*/
+			stream_jbuf_reset(au->strm, frames, frames*2);
+		}
+
+		sdp = stream_sdpmedia(au->strm);
+		sdp_media_del_lattr(sdp, "ebuacip");
+	}
+
+	return false;
+}
+
+
 int audio_alloc(struct audio **ap, const struct stream_param *stream_prm,
 		const struct config *cfg,
 		struct call *call, struct sdp_session *sdp_sess, int label,
@@ -1149,7 +1190,7 @@ int audio_alloc(struct audio **ap, const struct stream_param *stream_prm,
 
 	/* Audio codecs */
 	for (le = list_head(aucodecl); le; le = le->next) {
-		err = add_audio_codec(a, stream_sdpmedia(a->strm), le->data);
+		err = add_audio_codec(stream_sdpmedia(a->strm), le->data);
 		if (err)
 			goto out;
 	}
@@ -1246,7 +1287,8 @@ static void *tx_thread(void *arg)
 
 
 static void aufilt_param_set(struct aufilt_prm *prm,
-			     const struct aucodec *ac, uint32_t ptime)
+			     const struct aucodec *ac, uint32_t ptime,
+			     enum aufmt fmt)
 {
 	if (!ac) {
 		memset(prm, 0, sizeof(*prm));
@@ -1256,6 +1298,7 @@ static void aufilt_param_set(struct aufilt_prm *prm,
 	prm->srate      = get_srate(ac);
 	prm->ch         = get_ch(ac);
 	prm->ptime      = ptime;
+	prm->fmt        = fmt;
 }
 
 
@@ -1333,8 +1376,8 @@ static int aufilt_setup(struct audio *a)
 	if (!list_isempty(&tx->filtl) || !list_isempty(&rx->filtl))
 		return 0;
 
-	aufilt_param_set(&encprm, tx->ac, tx->ptime);
-	aufilt_param_set(&decprm, rx->ac, rx->ptime);
+	aufilt_param_set(&encprm, tx->ac, tx->ptime, tx->enc_fmt);
+	aufilt_param_set(&decprm, rx->ac, rx->ptime, rx->dec_fmt);
 
 	/* Audio filters */
 	for (le = list_head(baresip_aufiltl()); le; le = le->next) {
@@ -1344,21 +1387,27 @@ static int aufilt_setup(struct audio *a)
 		void *ctx = NULL;
 
 		if (af->encupdh) {
-			err |= af->encupdh(&encst, &ctx, af, &encprm, a);
-			if (err)
-				break;
-
-			encst->af = af;
-			list_append(&tx->filtl, &encst->le, encst);
+			err = af->encupdh(&encst, &ctx, af, &encprm, a);
+			if (err) {
+				warning("audio: error in encode audio-filter"
+					" '%s' (%m)\n", af->name, err);
+			}
+			else {
+				encst->af = af;
+				list_append(&tx->filtl, &encst->le, encst);
+			}
 		}
 
 		if (af->decupdh) {
-			err |= af->decupdh(&decst, &ctx, af, &decprm, a);
-			if (err)
-				break;
-
-			decst->af = af;
-			list_append(&rx->filtl, &decst->le, decst);
+			err = af->decupdh(&decst, &ctx, af, &decprm, a);
+			if (err) {
+				warning("audio: error in decode audio-filter"
+					" '%s' (%m)\n", af->name, err);
+			}
+			else {
+				decst->af = af;
+				list_append(&rx->filtl, &decst->le, decst);
+			}
 		}
 
 		if (err) {
@@ -1636,6 +1685,22 @@ void audio_stop(struct audio *a)
 }
 
 
+/**
+ * Check if audio has been started
+ *
+ * @param a Audio object
+ *
+ * @return True if audio has been started, otherwise false
+ */
+bool audio_started(const struct audio *a)
+{
+	if (!a)
+		return false;
+
+	return a->started;
+}
+
+
 int audio_encoder_set(struct audio *a, const struct aucodec *ac,
 		      int pt_tx, const char *params)
 {
@@ -1657,6 +1722,7 @@ int audio_encoder_set(struct audio *a, const struct aucodec *ac,
 		/* Audio source must be stopped first */
 		if (reset) {
 			tx->ausrc = mem_deref(tx->ausrc);
+			aubuf_flush(tx->aubuf);
 		}
 
 		tx->enc = mem_deref(tx->enc);
@@ -1726,6 +1792,7 @@ int audio_decoder_set(struct audio *a, const struct aucodec *ac,
 	if (reset) {
 
 		rx->auplay = mem_deref(rx->auplay);
+		aubuf_flush(rx->aubuf);
 
 		/* Reset audio filter chain */
 		list_flush(&rx->filtl);
@@ -1841,6 +1908,8 @@ static bool extmap_handler(const char *name, const char *value, void *arg)
 	int err;
 	(void)name;
 
+	MAGIC_CHECK(au);
+
 	err = sdp_extmap_decode(&extmap, value);
 	if (err) {
 		warning("audio: sdp_extmap_decode error (%m)\n", err);
@@ -1901,6 +1970,13 @@ void audio_sdp_attr_decode(struct audio *a)
 		}
 	}
 
+	/*
+	 * EBUACIP handler
+	 * EBU TECH 3368 profile provisioning on incoming invite.
+	 */
+	sdp_media_rattr_apply(stream_sdpmedia(a->strm), "ebuacip",
+			      ebuacip_handler, a);
+
 	/* Client-to-Mixer Audio Level Indication */
 	if (a->cfg.level) {
 		sdp_media_rattr_apply(stream_sdpmedia(a->strm),
@@ -1946,6 +2022,14 @@ static int aucodec_print(struct re_printf *pf, const struct aucodec *ac)
 }
 
 
+/**
+ * Print the audio debug information
+ *
+ * @param pf   Print function
+ * @param a    Audio object
+ *
+ * @return 0 if success, otherwise errorcode
+ */
 int audio_debug(struct re_printf *pf, const struct audio *a)
 {
 	const struct autx *tx;
