@@ -19,11 +19,21 @@ struct contact {
 	struct le he;          /* hash-element with key 'auri' */
 	struct sip_addr addr;
 	char *buf;
+	char *uri;
 	enum presence_status status;
 	enum access access;
 };
 
 
+struct contacts {
+	struct list cl;
+	struct hash *cht;
+
+	contact_update_h *handler;
+	void *handler_arg;
+};
+
+
 static void destructor(void *arg)
 {
 	struct contact *c = arg;
@@ -31,6 +41,17 @@ static void destructor(void *arg)
 	hash_unlink(&c->he);
 	list_unlink(&c->le);
 	mem_deref(c->buf);
+	mem_deref(c->uri);
+}
+
+
+static void contacts_destructor(void *data)
+{
+	struct contacts *contacts = data;
+
+	hash_clear(contacts->cht);
+	mem_deref(contacts->cht);
+	list_flush(&contacts->cl);
 }
 
 
@@ -69,6 +90,10 @@ int contact_add(struct contacts *contacts,
 		goto out;
 	}
 
+	err = pl_strdup(&c->uri, &c->addr.auri);
+	if (err)
+		goto out;
+
 	if (0 == msg_param_decode(&c->addr.params, "access", &pl)) {
 
 		if (0 == pl_strcasecmp(&pl, "block")) {
@@ -129,9 +154,8 @@ void contact_remove(struct contacts *contacts, struct contact *contact)
 void contact_set_update_handler(struct contacts *contacts,
 				contact_update_h *updateh, void *arg)
 {
-	if (!contacts) {
+	if (!contacts)
 		return;
-	}
 
 	contacts->handler = updateh;
 	contacts->handler_arg = arg;
@@ -164,6 +188,19 @@ const char *contact_str(const struct contact *c)
 }
 
 
+/**
+ * Get the SIP uri of a contact
+ *
+ * @param c Contact
+ *
+ * @return SIP uri
+ */
+const char *contact_uri(const struct contact *c)
+{
+	return c ? c->uri : NULL;
+}
+
+
 /**
  * Get the list of contacts
  *
@@ -195,6 +232,7 @@ void contact_set_presence(struct contact *c, enum presence_status status)
 	c->status = status;
 }
 
+
 enum presence_status contact_presence(const struct contact *c)
 {
 	if (!c)
@@ -203,6 +241,7 @@ enum presence_status contact_presence(const struct contact *c)
 	return c->status;
 }
 
+
 const char *contact_presence_str(enum presence_status status)
 {
 	switch (status) {
@@ -216,6 +255,15 @@ const char *contact_presence_str(enum presence_status status)
 }
 
 
+int contact_print(struct re_printf *pf, const struct contact *cnt)
+{
+	if (!cnt)
+		return 0;
+
+	return re_hprintf(pf, "%r <%r>", &cnt->addr.dname, &cnt->addr.auri);
+}
+
+
 int contacts_print(struct re_printf *pf, const struct contacts *contacts)
 {
 	const struct list *lst;
@@ -232,11 +280,10 @@ int contacts_print(struct re_printf *pf, const struct contacts *contacts)
 
 	for (le = list_head(lst); le && !err; le = le->next) {
 		const struct contact *c = le->data;
-		const struct sip_addr *addr = &c->addr;
 
-		err = re_hprintf(pf, "%20s  %r <%r>\n",
+		err = re_hprintf(pf, "%20s  %H\n",
 				 contact_presence_str(c->status),
-				 &addr->dname, &addr->auri);
+				 contact_print, c);
 	}
 
 	err |= re_hprintf(pf, "\n");
@@ -248,43 +295,38 @@ int contacts_print(struct re_printf *pf, const struct contacts *contacts)
 /**
  * Initialise the contacts sub-system
  *
- * @param contacts Contacts container
+ * @param contactsp Pointer to allocated contacts container
  *
  * @return 0 if success, otherwise errorcode
  */
-int contact_init(struct contacts *contacts)
+int contact_init(struct contacts **contactsp)
 {
+	struct contacts *contacts;
 	int err = 0;
 
-	if (!contacts)
+	if (!contactsp)
 		return EINVAL;
 
-	memset(contacts, 0, sizeof(*contacts));
+	contacts = mem_zalloc(sizeof(*contacts), contacts_destructor);
+	if (!contacts)
+		return ENOMEM;
 
 	list_init(&contacts->cl);
 
 	err = hash_alloc(&contacts->cht, 32);
+	if (err)
+		goto out;
+
+ out:
+	if (err)
+		mem_deref(contacts);
+	else
+		*contactsp = contacts;
 
 	return err;
 }
 
 
-/**
- * @param contacts Contacts container
- *
- * Close the contacts sub-system
- */
-void contact_close(struct contacts *contacts)
-{
-	if (!contacts)
-		return;
-
-	hash_clear(contacts->cht);
-	contacts->cht = mem_deref(contacts->cht);
-	list_flush(&contacts->cl);
-}
-
-
 static bool find_handler(struct le *le, void *arg)
 {
 	struct contact *c = le->data;
