@@ -30,6 +30,8 @@ struct ua {
 	int af_media;                /**< Preferred Address Family for media */
 	enum presence_status my_status; /**< Presence Status                 */
 	bool catchall;               /**< Catch all inbound requests         */
+	struct list hdr_filter;      /**< Filter for incoming headers        */
+	struct list custom_hdrs;     /**< List of outgoing headers           */
 };
 
 struct ua_eh {
@@ -38,6 +40,11 @@ struct ua_eh {
 	void *arg;
 };
 
+struct ua_xhdr_filter {
+	struct le le;
+	char *hdr_name;
+};
+
 static struct {
 	struct config_sip *cfg;        /**< SIP configuration               */
 	struct list ual;               /**< List of User-Agents (struct ua) */
@@ -81,13 +88,6 @@ static struct {
 };
 
 
-/* prototypes */
-static int ua_call_alloc(struct call **callp, struct ua *ua,
-			 enum vidmode vidmode, const struct sip_msg *msg,
-			 struct call *xcall, const char *local_uri,
-			 bool use_rtp);
-
-
 /* This function is called when all SIP transactions are done */
 static void exit_handler(void *arg)
 {
@@ -158,6 +158,10 @@ int ua_register(struct ua *ua)
 		return EINVAL;
 
 	acc = ua->acc;
+
+	if (acc->regint == 0)
+		return 0;
+
 	uri = ua->acc->luri;
 	uri.user = uri.password = pl_null;
 
@@ -237,6 +241,13 @@ void ua_unregister(struct ua *ua)
 }
 
 
+/**
+ * Check if a User-Agent is registered
+ *
+ * @param ua User-Agent object
+ *
+ * @return True if registered, false if not registered
+ */
 bool ua_isregistered(const struct ua *ua)
 {
 	struct le *le;
@@ -294,8 +305,6 @@ static void call_event_handler(struct call *call, enum call_event ev,
 {
 	struct ua *ua = arg;
 	const char *peeruri;
-	struct call *call2 = NULL;
-	int err;
 
 	MAGIC_CHECK(ua);
 
@@ -354,34 +363,7 @@ static void call_event_handler(struct call *call, enum call_event ev,
 		break;
 
 	case CALL_EVENT_TRANSFER:
-
-		/*
-		 * Create a new call to transfer target.
-		 *
-		 * NOTE: we will automatically connect a new call to the
-		 *       transfer target
-		 */
-
-		ua_printf(ua, "transferring call to %s\n", str);
-
-		err = ua_call_alloc(&call2, ua, VIDMODE_ON, NULL, call,
-				    call_localuri(call), true);
-		if (!err) {
-			struct pl pl;
-
-			pl_set_str(&pl, str);
-
-			err = call_connect(call2, &pl);
-			if (err) {
-				warning("ua: transfer: connect error: %m\n",
-					err);
-			}
-		}
-
-		if (err) {
-			(void)call_notify_sipfrag(call, 500, "Call Error");
-			mem_deref(call2);
-		}
+		ua_event(ua, UA_EVENT_CALL_TRANSFER, call, str);
 		break;
 
 	case CALL_EVENT_TRANSFER_FAILED:
@@ -415,20 +397,18 @@ static void call_dtmf_handler(struct call *call, char key, void *arg)
 }
 
 
-static int ua_call_alloc(struct call **callp, struct ua *ua,
-			 enum vidmode vidmode, const struct sip_msg *msg,
-			 struct call *xcall, const char *local_uri,
-			 bool use_rtp)
+int ua_call_alloc(struct call **callp, struct ua *ua,
+		  enum vidmode vidmode, const struct sip_msg *msg,
+		  struct call *xcall, const char *local_uri,
+		  bool use_rtp)
 {
 	const struct network *net = baresip_network();
 	struct call_prm cprm;
 	int af = AF_UNSPEC;
 	int err;
 
-	if (*callp) {
-		warning("ua: call_alloc: call is already allocated\n");
-		return EALREADY;
-	}
+	if (!callp || !ua)
+		return EINVAL;
 
 	/* 1. if AF_MEDIA is set, we prefer it
 	 * 2. otherwise fall back to SIP AF
@@ -508,7 +488,7 @@ static void handle_options(struct ua *ua, const struct sip_msg *msg)
 			  "Content-Length: %zu\r\n"
 			  "\r\n"
 			  "%b",
-			  uag_allowed_methods(),
+			  ua_allowed_methods(ua),
 			  ua_print_supported, ua,
 			  sip_contact_print, &contact,
 			  desc ? "Content-Type: application/sdp\r\n" : "",
@@ -543,6 +523,9 @@ static void ua_destructor(void *arg)
 	if (list_isempty(&uag.ual)) {
 		sip_close(uag.sip, false);
 	}
+
+	list_flush(&ua->custom_hdrs);
+	list_flush(&ua->hdr_filter);
 }
 
 
@@ -695,10 +678,6 @@ int ua_alloc(struct ua **uap, const char *aor)
 
 	list_append(&uag.ual, &ua->le, ua);
 
-	if (ua->acc->regint) {
-		err = ua_register(ua);
-	}
-
 	if (!uag_current())
 		uag_current_set(ua);
 
@@ -713,6 +692,13 @@ int ua_alloc(struct ua **uap, const char *aor)
 }
 
 
+/**
+ * Update a User-agent object, reset register clients
+ *
+ * @param ua User-Agent object
+ *
+ * @return 0 if success, otherwise errorcode
+ */
 int ua_update_account(struct ua *ua)
 {
 	if (!ua)
@@ -726,12 +712,18 @@ int ua_update_account(struct ua *ua)
 }
 
 
-static int uri_complete(struct ua *ua, struct mbuf *buf, const char *uri)
+int ua_uri_complete(struct ua *ua, struct mbuf *buf, const char *uri)
 {
-	size_t len;
-	int err = 0;
-	bool uri_is_ip;
+	struct account *acc;
 	struct sa sa_addr;
+	size_t len;
+	bool uri_is_ip;
+	int err = 0;
+
+	if (!ua || !buf || !uri)
+		return EINVAL;
+
+	acc = ua->acc;
 
 	/* Skip initial whitespace */
 	while (isspace(*uri))
@@ -753,23 +745,23 @@ static int uri_complete(struct ua *ua, struct mbuf *buf, const char *uri)
 	if (0 != re_regex(uri, len, "[^@]+@[^]+", NULL, NULL) &&
 		1 != uri_is_ip) {
 #if HAVE_INET6
-		if (AF_INET6 == ua->acc->luri.af)
+		if (AF_INET6 == acc->luri.af)
 			err |= mbuf_printf(buf, "@[%r]",
-					   &ua->acc->luri.host);
+					   &acc->luri.host);
 		else
 #endif
 			err |= mbuf_printf(buf, "@%r",
-					   &ua->acc->luri.host);
+					   &acc->luri.host);
 
 		/* Also append port if specified and not 5060 */
-		switch (ua->acc->luri.port) {
+		switch (acc->luri.port) {
 
 		case 0:
 		case SIP_PORT:
 			break;
 
 		default:
-			err |= mbuf_printf(buf, ":%u", ua->acc->luri.port);
+			err |= mbuf_printf(buf, ":%u", acc->luri.port);
 			break;
 		}
 	}
@@ -784,43 +776,32 @@ static int uri_complete(struct ua *ua, struct mbuf *buf, const char *uri)
  * @param ua        User-Agent
  * @param callp     Optional pointer to allocated call object
  * @param from_uri  Optional From uri, or NULL for default AOR
- * @param uri       SIP uri to connect to
- * @param params    Optional URI parameters
+ * @param req_uri   SIP uri to connect to
  * @param vmode     Video mode
  *
  * @return 0 if success, otherwise errorcode
  */
 int ua_connect(struct ua *ua, struct call **callp,
-	       const char *from_uri, const char *uri,
-	       const char *params, enum vidmode vmode)
+	       const char *from_uri, const char *req_uri,
+	       enum vidmode vmode)
 {
 	struct call *call = NULL;
 	struct mbuf *dialbuf;
 	struct pl pl;
 	int err = 0;
 
-	if (!ua || !str_isset(uri))
+	if (!ua || !str_isset(req_uri))
 		return EINVAL;
 
 	dialbuf = mbuf_alloc(64);
 	if (!dialbuf)
 		return ENOMEM;
 
-	if (params)
-		err |= mbuf_printf(dialbuf, "<");
-
-	err |= uri_complete(ua, dialbuf, uri);
-
-	if (params) {
-		err |= mbuf_printf(dialbuf, ";%s", params);
-	}
+	err |= ua_uri_complete(ua, dialbuf, req_uri);
 
 	/* Append any optional URI parameters */
 	err |= mbuf_write_pl(dialbuf, &ua->acc->luri.params);
 
-	if (params)
-		err |= mbuf_printf(dialbuf, ">");
-
 	if (err)
 		goto out;
 
@@ -831,6 +812,9 @@ int ua_connect(struct ua *ua, struct call **callp,
 	pl.p = (char *)dialbuf->buf;
 	pl.l = dialbuf->end;
 
+	if (!list_isempty(&ua->custom_hdrs))
+		call_set_custom_hdrs(call, &ua->custom_hdrs);
+
 	err = call_connect(call, &pl);
 
 	if (err)
@@ -899,6 +883,14 @@ int ua_answer(struct ua *ua, struct call *call)
 }
 
 
+/**
+ * Answer an incoming call with early media
+ *
+ * @param ua   User-Agent
+ * @param call Call to answer, or NULL for current call
+ *
+ * @return 0 if success, otherwise errorcode
+ */
 int ua_progress(struct ua *ua, struct call *call)
 {
 	if (!ua)
@@ -951,6 +943,14 @@ int ua_hold_answer(struct ua *ua, struct call *call)
 }
 
 
+/**
+ * Print the user-agent registration status
+ *
+ * @param pf  Print function
+ * @param ua  User-Agent object
+ *
+ * @return 0 if success, otherwise errorcode
+ */
 int ua_print_status(struct re_printf *pf, const struct ua *ua)
 {
 	struct le *le;
@@ -993,7 +993,7 @@ int ua_options_send(struct ua *ua, const char *uri,
 	if (!dialbuf)
 		return ENOMEM;
 
-	err = uri_complete(ua, dialbuf, uri);
+	err = ua_uri_complete(ua, dialbuf, uri);
 	if (err)
 		goto out;
 
@@ -1112,6 +1112,14 @@ struct call *ua_prev_call(const struct ua *ua)
 }
 
 
+/**
+ * Print the user-agent debug information
+ *
+ * @param pf  Print function
+ * @param ua  User-Agent object
+ *
+ * @return 0 if success, otherwise errorcode
+ */
 int ua_debug(struct re_printf *pf, const struct ua *ua)
 {
 	struct le *le;
@@ -1196,6 +1204,19 @@ static int add_transp_af(const struct sa *laddr)
 				warning("ua: tls_alloc() failed: %m\n", err);
 				return err;
 			}
+
+			if (str_isset(uag.cfg->cafile)) {
+				const char *ca = uag.cfg->cafile;
+
+				info("ua: adding SIP CA: %s\n", ca);
+
+				err = tls_add_ca(uag.tls, ca);
+				if (err) {
+					warning("ua: tls_add_ca() failed:"
+						" %m\n", err);
+					return err;
+				}
+			}
 		}
 
 		if (sa_isset(&local, SA_PORT))
@@ -1309,6 +1330,32 @@ static void sipsess_conn_handler(const struct sip_msg *msg, void *arg)
 		goto error;
 	}
 
+	if (!list_isempty(&ua->hdr_filter)) {
+		struct list hdrs;
+		struct le *le;
+
+		list_init(&hdrs);
+
+		le = list_head(&ua->hdr_filter);
+		while (le) {
+		    const struct sip_hdr *tmp_hdr;
+		    const struct ua_xhdr_filter *filter = le->data;
+
+		    le = le->next;
+		    tmp_hdr = sip_msg_xhdr(msg, filter->hdr_name);
+
+		    if (tmp_hdr) {
+		        char name[256];
+		        pl_strcpy(&tmp_hdr->name, name, sizeof(name));
+		        if (custom_hdrs_add(&hdrs, name, "%r", &tmp_hdr->val))
+					goto error;
+			}
+		}
+
+		call_set_custom_hdrs(call, &hdrs);
+		list_flush(&hdrs);
+	}
+
 	err = call_accept(call, uag.sock, msg);
 	if (err)
 		goto error;
@@ -1321,6 +1368,37 @@ static void sipsess_conn_handler(const struct sip_msg *msg, void *arg)
 }
 
 
+static void ua_xhdr_filter_destructor(void *arg)
+{
+	struct ua_xhdr_filter *filter = arg;
+	mem_deref(filter->hdr_name);
+}
+
+
+int ua_add_xhdr_filter(struct ua *ua, const char *hdr_name)
+{
+	struct ua_xhdr_filter *filter;
+
+	if (!ua)
+		return EINVAL;
+
+	filter = mem_zalloc(sizeof(*filter), ua_xhdr_filter_destructor);
+	if (!filter) {
+		return ENOMEM;
+	}
+
+	if (str_dup(&filter->hdr_name, hdr_name))
+	{
+		mem_deref(filter);
+		return ENOMEM;
+	}
+
+	list_append(&ua->hdr_filter, &filter->le, filter);
+
+	return 0;
+}
+
+
 static void net_change_handler(void *arg)
 {
 	(void)arg;
@@ -1453,7 +1531,7 @@ void ua_close(void)
 void ua_stop_all(bool forced)
 {
 	struct le *le;
-	bool ext_ref = false;
+	unsigned ext_ref = 0;
 
 	info("ua: stop all (forced=%d)\n", forced);
 
@@ -1464,36 +1542,28 @@ void ua_stop_all(bool forced)
 		struct ua *ua = le->data;
 		le = le->next;
 
+		ua_event(ua, UA_EVENT_SHUTDOWN, NULL, NULL);
+
 		if (mem_nrefs(ua) > 1) {
-
-			list_unlink(&ua->le);
-			list_flush(&ua->calls);
-			mem_deref(ua);
-
-			ext_ref = true;
+			++ext_ref;
 		}
 
-		ua_event(ua, UA_EVENT_SHUTDOWN, NULL, NULL);
+		list_unlink(&ua->le);
+		list_flush(&ua->calls);
+		mem_deref(ua);
 	}
 
 	if (ext_ref) {
-		info("ua: ext_ref -> cannot unload mods\n");
+		info("ua: in use (%u) by app module"
+		     ", deferring module unloading\n", ext_ref);
 		return;
 	}
 	else {
 		module_app_unload();
 	}
 
-	if (!list_isempty(&uag.ual)) {
-		const uint32_t n = list_count(&uag.ual);
-		info("Stopping %u useragent%s.. %s\n",
-		     n, n==1 ? "" : "s", forced ? "(Forced)" : "");
-	}
-
 	if (forced)
 		sipsess_close_all(uag.sock);
-	else
-		list_flush(&uag.ual);
 
 	sip_close(uag.sip, forced);
 }
@@ -1786,6 +1856,13 @@ const char *ua_cuser(const struct ua *ua)
 }
 
 
+/**
+ * Get the local contact username
+ *
+ * @param ua User-Agent
+ *
+ * @return Local contact username
+ */
 const char *ua_local_cuser(const struct ua *ua)
 {
 	return ua ? ua->cuser : NULL;
@@ -1821,6 +1898,11 @@ void ua_pub_gruu_set(struct ua *ua, const struct pl *pval)
 }
 
 
+/**
+ * Get the list of User-Agents
+ *
+ * @return List of User-Agents (struct ua)
+ */
 struct list *uag_list(void)
 {
 	return &uag.ual;
@@ -1830,15 +1912,27 @@ struct list *uag_list(void)
 /**
  * Return list of methods supported by the UA
  *
+ * @param ua User-Agent
+ *
  * @return String of supported methods
  */
-const char *uag_allowed_methods(void)
+const char *ua_allowed_methods(const struct ua *ua)
 {
-	return "INVITE,ACK,BYE,CANCEL,OPTIONS,REFER,"
+	return ua->acc->refer ? "INVITE,ACK,BYE,CANCEL,OPTIONS,"
+		"NOTIFY,SUBSCRIBE,INFO,MESSAGE,REFER" :
+		"INVITE,ACK,BYE,CANCEL,OPTIONS,"
 		"NOTIFY,SUBSCRIBE,INFO,MESSAGE";
 }
 
 
+/**
+ * Print the supported extensions
+ *
+ * @param pf  Print function
+ * @param ua  User-Agent object
+ *
+ * @return 0 if success, otherwise errorcode
+ */
 int ua_print_supported(struct re_printf *pf, const struct ua *ua)
 {
 	size_t i;
@@ -1857,6 +1951,13 @@ int ua_print_supported(struct re_printf *pf, const struct ua *ua)
 }
 
 
+/**
+ * Get the list of call objects
+ *
+ * @param ua User-Agent
+ *
+ * @return List of call objects (struct call)
+ */
 struct list *ua_calls(const struct ua *ua)
 {
 	return ua ? (struct list *)&ua->calls : NULL;
@@ -1870,6 +1971,14 @@ static void eh_destructor(void *arg)
 }
 
 
+/**
+ * Register a User-Agent event handler
+ *
+ * @param h   Event handler
+ * @param arg Handler argument
+ *
+ * @return 0 if success, otherwise errorcode
+ */
 int uag_event_register(ua_event_h *h, void *arg)
 {
 	struct ua_eh *eh;
@@ -1892,6 +2001,11 @@ int uag_event_register(ua_event_h *h, void *arg)
 }
 
 
+/**
+ * Unregister a User-Agent event handler
+ *
+ * @param h   Event handler
+ */
 void uag_event_unregister(ua_event_h *h)
 {
 	struct le *le;
@@ -1908,18 +2022,33 @@ void uag_event_unregister(ua_event_h *h)
 }
 
 
+/**
+ * Set the handler to receive incoming SIP SUBSCRIBE messages
+ *
+ * @param subh Subscribe handler
+ */
 void uag_set_sub_handler(sip_msg_h *subh)
 {
 	uag.subh = subh;
 }
 
 
+/**
+ * Set the current User-Agent
+ *
+ * @param ua User-Agent to set as current
+ */
 void uag_current_set(struct ua *ua)
 {
 	uag.ua_cur = ua;
 }
 
 
+/**
+ * Get the current User-Agent
+ *
+ * @return Current User-Agent, NULL if none
+ */
 struct ua *uag_current(void)
 {
 	if (list_isempty(uag_list()))
@@ -1929,6 +2058,12 @@ struct ua *uag_current(void)
 }
 
 
+/**
+ * Set the preferred address family for media
+ *
+ * @param ua       User-Agent
+ * @param af_media Address family (e.g. AF_INET, AF_INET6)
+ */
 void ua_set_media_af(struct ua *ua, int af_media)
 {
 	if (!ua)
@@ -1954,6 +2089,13 @@ void ua_set_catchall(struct ua *ua, bool enabled)
 }
 
 
+/**
+ * Set extra parameters to use for all SIP Accounts
+ *
+ * @param eprm Extra parameters
+ *
+ * @return 0 if success, otherwise errorcode
+ */
 int uag_set_extra_params(const char *eprm)
 {
 	uag.eprm = mem_deref(uag.eprm);
@@ -1963,3 +2105,23 @@ int uag_set_extra_params(const char *eprm)
 
 	return 0;
 }
+
+
+void ua_set_custom_hdrs(struct ua *ua, struct list *custom_headers)
+{
+	struct le *le;
+
+	if (!ua)
+		return;
+
+	list_flush(&ua->custom_hdrs);
+
+	LIST_FOREACH(custom_headers, le) {
+		struct sip_hdr *hdr = le->data;
+		char *buf = NULL;
+
+		re_sdprintf(&buf, "%r", &hdr->name);
+		custom_hdrs_add(&ua->custom_hdrs, buf, "%r", &hdr->val);
+		mem_deref(buf);
+	}
+}
