@@ -141,8 +141,7 @@ struct vrx {
 	double efps;                       /**< Estimated frame-rate      */
 	unsigned n_intra;                  /**< Intra-frames decoded      */
 	unsigned n_picup;                  /**< Picture updates sent      */
-	uint32_t ts_min;
-	uint32_t ts_max;
+	struct timestamp_recv ts_recv;     /**< Receive timestamp state   */
 
 	/** Statistics */
 	struct {
@@ -354,6 +353,8 @@ static int packet_handler(bool marker, uint64_t ts,
 	uint32_t rtp_ts;
 	int err;
 
+	MAGIC_CHECK(vtx->video);
+
 	if (!vtx->ts_base)
 		vtx->ts_base = ts;
 	vtx->ts_last = ts;
@@ -429,7 +430,7 @@ static void encode_rtp_send(struct vtx *vtx, struct vidframe *frame,
 		struct vidfilt_enc_st *st = le->data;
 
 		if (st->vf && st->vf->ench)
-			err |= st->vf->ench(st, frame);
+			err |= st->vf->ench(st, frame, &timestamp);
 	}
 
 	if (err)
@@ -461,8 +462,7 @@ static void vidsrc_frame_handler(struct vidframe *frame, uint64_t timestamp,
 {
 	struct vtx *vtx = arg;
 
-	/* XXX: save timestamp(s) and pass to encoder */
-	(void)timestamp;
+	MAGIC_CHECK(vtx->video);
 
 	++vtx->frames;
 
@@ -485,6 +485,8 @@ static void vidsrc_error_handler(int err, void *arg)
 {
 	struct vtx *vtx = arg;
 
+	MAGIC_CHECK(vtx->video);
+
 	warning("video: video-source error: %m\n", err);
 
 	vtx->vsrc = mem_deref(vtx->vsrc);
@@ -529,7 +531,6 @@ static int vrx_alloc(struct vrx *vrx, struct video *video)
 
 	str_ncpy(vrx->device, video->cfg.disp_dev, sizeof(vrx->device));
 
-	vrx->ts_min = ~0;
 	vrx->fmt = (enum vidfmt)-1;
 
 	return err;
@@ -540,6 +541,8 @@ static void picup_tmr_handler(void *arg)
 {
 	struct vrx *vrx = arg;
 
+	MAGIC_CHECK(vrx->video);
+
 	request_picture_update(vrx);
 }
 
@@ -562,6 +565,41 @@ static void request_picture_update(struct vrx *vrx)
 }
 
 
+static void update_rtp_timestamp(struct timestamp_recv *tsr, uint32_t rtp_ts)
+{
+	int wrap;
+
+	if (tsr->is_set) {
+
+		wrap = timestamp_wrap(rtp_ts, tsr->last);
+
+		switch (wrap) {
+
+		case -1:
+			info("video: rtp timestamp wraps backwards"
+			     " (delta = %d) -- discard\n",
+			     (int32_t)(tsr->last - rtp_ts));
+			return;
+
+		case 0:
+			break;
+
+		case 1:
+			++tsr->num_wraps;
+			break;
+
+		default:
+			break;
+		}
+	}
+	else {
+		timestamp_set(tsr, rtp_ts);
+	}
+
+	tsr->last = rtp_ts;
+}
+
+
 /**
  * Decode incoming RTP packets using the Video decoder
  *
@@ -580,6 +618,7 @@ static int video_stream_decode(struct vrx *vrx, const struct rtp_header *hdr,
 	struct vidframe *frame_filt = NULL;
 	struct vidframe frame_store, *frame = &frame_store;
 	struct le *le;
+	uint64_t timestamp;
 	bool intra;
 	int err = 0;
 
@@ -594,12 +633,12 @@ static int video_stream_decode(struct vrx *vrx, const struct rtp_header *hdr,
 		goto out;
 	}
 
-	/* todo: check if RTP timestamp wraps */
+	update_rtp_timestamp(&vrx->ts_recv, hdr->ts);
 
-	if (hdr->ts < vrx->ts_min)
-		vrx->ts_min = hdr->ts;
-	if (hdr->ts > vrx->ts_max)
-		vrx->ts_max = hdr->ts;
+	/* convert the RTP timestamp to VIDEO_TIMEBASE timestamp */
+	timestamp = video_calc_timebase_timestamp(
+			  timestamp_calc_extended(vrx->ts_recv.num_wraps,
+						  vrx->ts_recv.last));
 
 	frame->data[0] = NULL;
 	err = vrx->vc->dech(vrx->dec, frame, &intra, hdr->m, hdr->seq, mb);
@@ -646,12 +685,12 @@ static int video_stream_decode(struct vrx *vrx, const struct rtp_header *hdr,
 		struct vidfilt_dec_st *st = le->data;
 
 		if (st->vf && st->vf->dech)
-			err |= st->vf->dech(st, frame);
+			err |= st->vf->dech(st, frame, &timestamp);
 	}
 
 	++vrx->stats.disp_frames;
 
-	err = vidisp_display(vrx->vidisp, v->peer, frame);
+	err = vidisp_display(vrx->vidisp, v->peer, frame, timestamp);
 	frame_filt = mem_deref(frame_filt);
 	if (err == ENODEV) {
 		warning("video: video-display was closed\n");
@@ -675,7 +714,7 @@ out:
 }
 
 
-static int pt_handler(struct video *v, uint8_t pt_old, uint8_t pt_new)
+static int update_payload_type(struct video *v, uint8_t pt_old, uint8_t pt_new)
 {
 	const struct sdp_format *lc;
 
@@ -704,6 +743,8 @@ static void stream_recv_handler(const struct rtp_header *hdr,
 	(void)extv;
 	(void)extc;
 
+	MAGIC_CHECK(v);
+
 	if (!mb)
 		goto out;
 
@@ -711,7 +752,7 @@ static void stream_recv_handler(const struct rtp_header *hdr,
 	if (hdr->pt == v->vrx.pt_rx)
 		goto out;
 
-	err = pt_handler(v, v->vrx.pt_rx, hdr->pt);
+	err = update_payload_type(v, v->vrx.pt_rx, hdr->pt);
 	if (err)
 		return;
 
@@ -724,6 +765,8 @@ static void rtcp_handler(struct rtcp_msg *msg, void *arg)
 {
 	struct video *v = arg;
 
+	MAGIC_CHECK(v);
+
 	switch (msg->hdr.pt) {
 
 	case RTCP_FIR:
@@ -903,6 +946,8 @@ static void vidisp_resize_handler(const struct vidsz *sz, void *arg)
 	struct vrx *vrx = arg;
 	(void)vrx;
 
+	MAGIC_CHECK(vrx->video);
+
 	info("video: display resized: %u x %u\n", sz->w, sz->h);
 
 	/* XXX: update wanted picturesize and send re-invite to peer */
@@ -969,6 +1014,8 @@ static void tmr_handler(void *arg)
 {
 	struct video *v = arg;
 
+	MAGIC_CHECK(v);
+
 	tmr_start(&v->tmr, TMR_INTERVAL * 1000, tmr_handler, v);
 
 	/* Estimate framerates */
@@ -1261,6 +1308,13 @@ void video_encoder_cycle(struct video *video)
 }
 
 
+/**
+ * Get the RTP Stream object from a Video object
+ *
+ * @param v  Video object
+ *
+ * @return RTP Stream object
+ */
 struct stream *video_strm(const struct video *v)
 {
 	return v ? v->strm : NULL;
@@ -1365,13 +1419,27 @@ static int vrx_debug(struct re_printf *pf, const struct vrx *vrx)
 			  vrx->stats.disp_frames);
 	err |= re_hprintf(pf, "     n_intra=%u, n_picup=%u\n",
 			  vrx->n_intra, vrx->n_picup);
-	err |= re_hprintf(pf, "     time = %.3f sec\n",
-			  video_calc_seconds(vrx->ts_max - vrx->ts_min));
+
+	if (vrx->ts_recv.is_set) {
+		err |= re_hprintf(pf, "     time = %.3f sec\n",
+		  video_calc_seconds(timestamp_duration(&vrx->ts_recv)));
+	}
+	else {
+		err |= re_hprintf(pf, "     time = (not started)\n");
+	}
 
 	return err;
 }
 
 
+/**
+ * Print the video debug information
+ *
+ * @param pf   Print function
+ * @param v    Video object
+ *
+ * @return 0 if success, otherwise errorcode
+ */
 int video_debug(struct re_printf *pf, const struct video *v)
 {
 	const struct vtx *vtx;
@@ -1412,6 +1480,15 @@ int video_print(struct re_printf *pf, const struct video *v)
 }
 
 
+/**
+ * Set the active video source
+ *
+ * @param v    Video object
+ * @param name Video source name
+ * @param dev  Video source device
+ *
+ * @return 0 if success, otherwise errorcode
+ */
 int video_set_source(struct video *v, const char *name, const char *dev)
 {
 	struct vidsrc *vs = (struct vidsrc *)vidsrc_find(baresip_vidsrcl(),
@@ -1434,6 +1511,13 @@ int video_set_source(struct video *v, const char *name, const char *dev)
 }
 
 
+/**
+ * Set the device name of video source and display
+ *
+ * @param v    Video object
+ * @param src  Video source device
+ * @param disp Video display device
+ */
 void video_set_devicename(struct video *v, const char *src, const char *disp)
 {
 	if (!v)
