@@ -16,14 +16,16 @@
 enum behaviour {
 	BEHAVIOUR_ANSWER = 0,
 	BEHAVIOUR_PROGRESS,
-	BEHAVIOUR_REJECT
+	BEHAVIOUR_REJECT,
+	BEHAVIOUR_GET_HDRS,
 };
 
 enum action {
 	ACTION_RECANCEL = 0,
 	ACTION_HANGUP_A,
 	ACTION_HANGUP_B,
-	ACTION_NOTHING
+	ACTION_NOTHING,
+	ACTION_TRANSFER
 };
 
 struct agent {
@@ -38,15 +40,19 @@ struct agent {
 	unsigned n_established;
 	unsigned n_closed;
 	unsigned n_dtmf_recv;
+	unsigned n_transfer;
+	unsigned n_mediaenc;
 };
 
 struct fixture {
 	uint32_t magic;
-	struct agent a, b;
-	struct sa laddr_sip;
+	struct agent a, b, c;
+	struct sa laddr_udp;
+	struct sa laddr_tcp;
 	enum behaviour behaviour;
 	enum action estab_action;
 	char buri[256];
+	char buri_tcp[256];
 	int err;
 	unsigned exp_estab;
 	unsigned exp_closed;
@@ -58,8 +64,9 @@ struct fixture {
 									\
 	f->a.fix = f;							\
 	f->b.fix = f;							\
+	f->c.fix = f;							\
 									\
-	err = ua_init("test", true, true, true, false);			\
+	err = ua_init("test", true, true, false, false);		\
 	TEST_ERR(err);							\
 									\
 	f->magic = MAGIC;						\
@@ -80,11 +87,21 @@ struct fixture {
 	err = uag_event_register(event_handler, f);			\
 	TEST_ERR(err);							\
 									\
-	err = sip_transp_laddr(uag_sip(), &f->laddr_sip,		\
+	err = sip_transp_laddr(uag_sip(), &f->laddr_udp,		\
 			       SIP_TRANSP_UDP, NULL);			\
 	TEST_ERR(err);							\
 									\
-	re_snprintf(f->buri, sizeof(f->buri), "sip:b@%J", &f->laddr_sip);
+	err = sip_transp_laddr(uag_sip(), &f->laddr_tcp,		\
+			       SIP_TRANSP_TCP, NULL);			\
+	TEST_ERR(err);							\
+									\
+	debug("test: local SIP transp: UDP=%J, TCP=%J\n",		\
+	      &f->laddr_udp, &f->laddr_tcp);				\
+									\
+	re_snprintf(f->buri, sizeof(f->buri),				\
+		    "sip:b@%J", &f->laddr_udp);				\
+	re_snprintf(f->buri_tcp, sizeof(f->buri_tcp),			\
+		    "sip:b@%J;transport=tcp", &f->laddr_tcp);
 
 
 #define fixture_init(f)				\
@@ -92,6 +109,7 @@ struct fixture {
 
 
 #define fixture_close(f)			\
+	mem_deref(f->c.ua);			\
 	mem_deref(f->b.ua);			\
 	mem_deref(f->a.ua);			\
 						\
@@ -108,12 +126,16 @@ struct fixture {
 		re_cancel();			\
 	} while (0)
 
+static const struct list *hdrs;
+
 
 static void event_handler(struct ua *ua, enum ua_event ev,
 			  struct call *call, const char *prm, void *arg)
 {
 	struct fixture *f = arg;
+	struct call *call2 = NULL;
 	struct agent *ag;
+	char curi[256];
 	int err = 0;
 	(void)prm;
 
@@ -129,6 +151,8 @@ static void event_handler(struct ua *ua, enum ua_event ev,
 		ag = &f->a;
 	else if (ua == f->b.ua)
 		ag = &f->b;
+	else if (ua == f->c.ua)
+		ag = &f->c;
 	else {
 		return;
 	}
@@ -162,6 +186,15 @@ static void event_handler(struct ua *ua, enum ua_event ev,
 			ag->failed = true;
 			break;
 
+		case BEHAVIOUR_GET_HDRS:
+			hdrs = call_get_custom_hdrs(call);
+			err = ua_answer(ua, call);
+			if (err) {
+				warning("ua_answer failed (%m)\n", err);
+				goto out;
+			}
+			break;
+
 		default:
 			break;
 		}
@@ -201,6 +234,17 @@ static void event_handler(struct ua *ua, enum ua_event ev,
 			case ACTION_NOTHING:
 				/* Do nothing, wait */
 				break;
+
+			case ACTION_TRANSFER:
+				f->estab_action = ACTION_NOTHING;
+
+				re_snprintf(curi, sizeof(curi),
+					    "sip:c@%J", &f->laddr_udp);
+
+				err = call_transfer(ua_call(f->a.ua), curi);
+				if (err)
+					goto out;
+				break;
 			}
 		}
 		break;
@@ -220,6 +264,33 @@ static void event_handler(struct ua *ua, enum ua_event ev,
 		}
 		break;
 
+	case UA_EVENT_CALL_TRANSFER:
+		++ag->n_transfer;
+
+		err = ua_call_alloc(&call2, ua, VIDMODE_ON, NULL, call,
+				    call_localuri(call), true);
+		if (!err) {
+			struct pl pl;
+
+			pl_set_str(&pl, prm);
+
+			err = call_connect(call2, &pl);
+			if (err) {
+				warning("ua: transfer: connect error: %m\n",
+					err);
+			}
+		}
+
+		if (err) {
+			(void)call_notify_sipfrag(call, 500, "Call Error");
+			mem_deref(call2);
+		}
+		break;
+
+	case UA_EVENT_CALL_MENC:
+		++ag->n_mediaenc;
+		break;
+
 	default:
 		break;
 	}
@@ -249,7 +320,7 @@ int test_call_answer(void)
 	f->behaviour = BEHAVIOUR_ANSWER;
 
 	/* Make a call from A to B */
-	err = ua_connect(f->a.ua, 0, NULL, f->buri, NULL, VIDMODE_OFF);
+	err = ua_connect(f->a.ua, 0, NULL, f->buri, VIDMODE_OFF);
 	TEST_ERR(err);
 
 	/* run main-loop with timeout, wait for events */
@@ -283,7 +354,7 @@ int test_call_reject(void)
 	f->behaviour = BEHAVIOUR_REJECT;
 
 	/* Make a call from A to B */
-	err = ua_connect(f->a.ua, 0, NULL, f->buri, NULL, VIDMODE_OFF);
+	err = ua_connect(f->a.ua, 0, NULL, f->buri, VIDMODE_OFF);
 	TEST_ERR(err);
 
 	/* run main-loop with timeout, wait for events */
@@ -316,7 +387,7 @@ int test_call_af_mismatch(void)
 	ua_set_media_af(f->b.ua, AF_INET);
 
 	/* Make a call from A to B */
-	err = ua_connect(f->a.ua, 0, NULL, f->buri, NULL, VIDMODE_OFF);
+	err = ua_connect(f->a.ua, 0, NULL, f->buri, VIDMODE_OFF);
 	TEST_ERR(err);
 
 	/* run main-loop with timeout, wait for events */
@@ -351,7 +422,7 @@ int test_call_answer_hangup_a(void)
 	f->estab_action = ACTION_HANGUP_A;
 
 	/* Make a call from A to B */
-	err = ua_connect(f->a.ua, 0, NULL, f->buri, NULL, VIDMODE_OFF);
+	err = ua_connect(f->a.ua, 0, NULL, f->buri, VIDMODE_OFF);
 	TEST_ERR(err);
 
 	/* run main-loop with timeout, wait for events */
@@ -377,6 +448,7 @@ int test_call_answer_hangup_a(void)
 int test_call_answer_hangup_b(void)
 {
 	struct fixture fix, *f = &fix;
+	char uri[256];
 	int err = 0;
 
 	fixture_init(f);
@@ -384,8 +456,11 @@ int test_call_answer_hangup_b(void)
 	f->behaviour = BEHAVIOUR_ANSWER;
 	f->estab_action = ACTION_HANGUP_B;
 
+	/* add angle brackets */
+	re_snprintf(uri, sizeof(uri), "<%s>", f->buri);
+
 	/* Make a call from A to B */
-	err = ua_connect(f->a.ua, 0, NULL, f->buri, NULL, VIDMODE_OFF);
+	err = ua_connect(f->a.ua, 0, NULL, uri, VIDMODE_OFF);
 	TEST_ERR(err);
 
 	/* run main-loop with timeout, wait for events */
@@ -421,7 +496,7 @@ int test_call_rtp_timeout(void)
 	f->estab_action = ACTION_NOTHING;
 
 	/* Make a call from A to B */
-	err = ua_connect(f->a.ua, 0, NULL, f->buri, NULL, VIDMODE_OFF);
+	err = ua_connect(f->a.ua, 0, NULL, f->buri, VIDMODE_OFF);
 	TEST_ERR(err);
 
 	call = ua_call(f->a.ua);
@@ -484,7 +559,7 @@ int test_call_multiple(void)
 	 * Step 1 -- make 4 calls from A to B
 	 */
 	for (i=0; i<4; i++) {
-		err = ua_connect(f->a.ua, 0, NULL, f->buri, NULL, VIDMODE_OFF);
+		err = ua_connect(f->a.ua, 0, NULL, f->buri, VIDMODE_OFF);
 		TEST_ERR(err);
 	}
 
@@ -540,7 +615,7 @@ int test_call_multiple(void)
 	f->b.n_established = 0;
 	f->exp_estab = 2;
 	for (i=0; i<2; i++) {
-		err = ua_connect(f->a.ua, 0, NULL, f->buri, NULL, VIDMODE_OFF);
+		err = ua_connect(f->a.ua, 0, NULL, f->buri, VIDMODE_OFF);
 		TEST_ERR(err);
 	}
 
@@ -573,7 +648,7 @@ int test_call_max(void)
 
 	/* Make 2 calls, one should work and one should fail */
 	for (i=0; i<2; i++) {
-		err = ua_connect(f->a.ua, 0, NULL, f->buri, NULL, VIDMODE_OFF);
+		err = ua_connect(f->a.ua, 0, NULL, f->buri, VIDMODE_OFF);
 		TEST_ERR(err);
 	}
 
@@ -594,6 +669,9 @@ int test_call_max(void)
  out:
 	fixture_close(f);
 
+	/* Set the max-calls limit */
+	conf_config()->call.max_calls = 0;
+
 	return err;
 }
 
@@ -642,7 +720,7 @@ int test_call_dtmf(void)
 	f->behaviour = BEHAVIOUR_ANSWER;
 
 	/* Make a call from A to B */
-	err = ua_connect(f->a.ua, 0, NULL, f->buri, NULL, VIDMODE_OFF);
+	err = ua_connect(f->a.ua, 0, NULL, f->buri, VIDMODE_OFF);
 	TEST_ERR(err);
 
 	/* run main-loop with timeout, wait for events */
@@ -676,6 +754,19 @@ int test_call_dtmf(void)
 
 
 #ifdef USE_VIDEO
+static void mock_vidisp_handler(const struct vidframe *frame,
+				uint64_t timestamp, void *arg)
+{
+	struct fixture *fix = arg;
+	(void)frame;
+	(void)timestamp;
+	(void)fix;
+
+	/* Stop the test */
+	re_cancel();
+}
+
+
 int test_call_video(void)
 {
 	struct fixture fix, *f = &fix;
@@ -691,14 +782,14 @@ int test_call_video(void)
 	mock_vidcodec_register();
 	err = mock_vidsrc_register(&vidsrc);
 	TEST_ERR(err);
-	err = mock_vidisp_register(&vidisp);
+	err = mock_vidisp_register(&vidisp, mock_vidisp_handler, f);
 	TEST_ERR(err);
 
 	f->behaviour = BEHAVIOUR_ANSWER;
 	f->estab_action = ACTION_NOTHING;
 
 	/* Make a call from A to B */
-	err = ua_connect(f->a.ua, 0, NULL, f->buri, NULL, VIDMODE_ON);
+	err = ua_connect(f->a.ua, 0, NULL, f->buri, VIDMODE_ON);
 	TEST_ERR(err);
 
 	/* run main-loop with timeout, wait for events */
@@ -761,7 +852,7 @@ int test_call_aulevel(void)
 	f->estab_action = ACTION_NOTHING;
 
 	/* Make a call from A to B */
-	err = ua_connect(f->a.ua, 0, NULL, f->buri, NULL, VIDMODE_OFF);
+	err = ua_connect(f->a.ua, 0, NULL, f->buri, VIDMODE_OFF);
 	TEST_ERR(err);
 
 	/* run main-loop with timeout, wait for events */
@@ -798,7 +889,7 @@ int test_call_progress(void)
 	f->behaviour = BEHAVIOUR_PROGRESS;
 
 	/* Make a call from A to B */
-	err = ua_connect(f->a.ua, 0, NULL, f->buri, NULL, VIDMODE_OFF);
+	err = ua_connect(f->a.ua, 0, NULL, f->buri, VIDMODE_OFF);
 	TEST_ERR(err);
 
 	/* run main-loop with timeout, wait for events */
@@ -866,7 +957,7 @@ static int test_media_base(enum audio_mode txmode)
 	f->behaviour = BEHAVIOUR_ANSWER;
 
 	/* Make a call from A to B */
-	err = ua_connect(f->a.ua, 0, NULL, f->buri, NULL, VIDMODE_OFF);
+	err = ua_connect(f->a.ua, 0, NULL, f->buri, VIDMODE_OFF);
 	TEST_ERR(err);
 
 	/* run main-loop with timeout, wait for events */
@@ -941,7 +1032,7 @@ int test_call_mediaenc(void)
 	f->behaviour = BEHAVIOUR_ANSWER;
 
 	/* Make a call from A to B */
-	err = ua_connect(f->a.ua, 0, NULL, f->buri, NULL, VIDMODE_OFF);
+	err = ua_connect(f->a.ua, 0, NULL, f->buri, VIDMODE_OFF);
 	TEST_ERR(err);
 
 	/* run main-loop with timeout, wait for events */
@@ -955,7 +1046,9 @@ int test_call_mediaenc(void)
 	ASSERT_EQ(1, fix.b.n_established);
 	ASSERT_EQ(0, fix.b.n_closed);
 
-	/* XXX: verify that the call was encrypted */
+	/* verify that the call was encrypted */
+	ASSERT_EQ(1, fix.a.n_mediaenc);
+	ASSERT_EQ(1, fix.b.n_mediaenc);
 
  out:
 	fixture_close(f);
@@ -969,3 +1062,162 @@ int test_call_mediaenc(void)
 
 	return err;
 }
+
+
+int test_call_custom_headers(void)
+{
+	struct fixture fix, *f = &fix;
+	int err = 0;
+	int some_id = 7;
+	struct list custom_hdrs;
+	bool headers_matched = true;
+
+	fixture_init(f);
+
+	ua_add_xhdr_filter(f->b.ua, "X-CALL_ID");
+	ua_add_xhdr_filter(f->b.ua, "X-HEADER_NAME");
+
+	f->behaviour = BEHAVIOUR_GET_HDRS;
+
+	/* Make a call from A to B
+	 * with some custom headers in INVITE message */
+
+	list_init(&custom_hdrs);
+	err = custom_hdrs_add(&custom_hdrs, "X-CALL_ID", "%d", some_id);
+	err = custom_hdrs_add(&custom_hdrs, "X-HEADER_NAME", "%s", "VALUE");
+	ua_set_custom_hdrs(f->a.ua, &custom_hdrs);
+	err = ua_connect(f->a.ua, 0, NULL, f->buri, VIDMODE_OFF);
+
+	list_flush(&custom_hdrs);
+	TEST_ERR(err);
+
+	/* run main-loop with timeout, wait for events */
+	err = re_main_timeout(5000);
+
+	if (!list_isempty(hdrs)) {
+		struct le *le;
+		for (le = list_head(hdrs); le; le = le->next) {
+		    struct sip_hdr *hdr = le->data;
+		    if (pl_strcasecmp(&hdr->name, "X-CALL_ID") == 0) {
+		        char buf[20];
+		        re_snprintf(buf, sizeof(buf), "%d", some_id);
+		        if (pl_strcasecmp(&hdr->val, buf) != 0) {
+		            headers_matched = false;
+		        }
+		    }
+		    if (pl_strcasecmp(&hdr->name, "X-HEADER_NAME") == 0) {
+		        if (pl_strcasecmp(&hdr->val, "VALUE") != 0) {
+		            headers_matched = false;
+		        }
+		    }
+		}
+	}
+	else {
+		headers_matched = false;
+	}
+
+	ASSERT_TRUE(headers_matched);
+
+	TEST_ERR(err);
+	TEST_ERR(fix.err);
+
+	ASSERT_EQ(0, fix.a.n_incoming);
+	ASSERT_EQ(1, fix.a.n_established);
+	ASSERT_EQ(0, fix.a.n_closed);
+	ASSERT_EQ(0, fix.a.close_scode);
+
+	ASSERT_EQ(1, fix.b.n_incoming);
+	ASSERT_EQ(1, fix.b.n_established);
+	ASSERT_EQ(0, fix.b.n_closed);
+
+ out:
+	fixture_close(f);
+
+	return err;
+}
+
+
+int test_call_tcp(void)
+{
+	struct fixture fix, *f = &fix;
+	int err = 0;
+
+	fixture_init(f);
+
+	f->behaviour = BEHAVIOUR_ANSWER;
+
+	/* Make a call using TCP-transport */
+	err = ua_connect(f->a.ua, 0, NULL, f->buri_tcp, VIDMODE_OFF);
+	TEST_ERR(err);
+
+	/* run main-loop with timeout, wait for events */
+	err = re_main_timeout(5000);
+	TEST_ERR(err);
+	TEST_ERR(fix.err);
+
+	ASSERT_EQ(1, fix.a.n_established);
+	ASSERT_EQ(1, fix.b.n_established);
+
+ out:
+	fixture_close(f);
+
+	return err;
+}
+
+
+/*
+ *  Step 1. Call from A to B
+ *
+ *  Step 2. A transfer B to C
+ *
+ *  Step 3. Call between B and C
+ *          No call for A
+ */
+int test_call_transfer(void)
+{
+	struct fixture fix, *f = &fix;
+	int err = 0;
+
+	fixture_init(f);
+
+	/* Create a 3rd useragent needed for transfer */
+	err = ua_alloc(&f->c.ua, "C <sip:c@127.0.0.1>;regint=0");
+	TEST_ERR(err);
+
+	f->c.peer = &f->b;
+
+	f->behaviour = BEHAVIOUR_ANSWER;
+	f->estab_action = ACTION_TRANSFER;
+
+	err = ua_connect(f->a.ua, 0, NULL, f->buri, VIDMODE_OFF);
+	TEST_ERR(err);
+
+	/* run main-loop with timeout, wait for events */
+	err = re_main_timeout(5000);
+	TEST_ERR(err);
+	TEST_ERR(fix.err);
+
+	ASSERT_EQ(0, fix.a.n_incoming);
+	ASSERT_EQ(1, fix.a.n_established);
+	ASSERT_EQ(1, fix.a.n_closed);
+	ASSERT_EQ(0, fix.a.n_transfer);
+
+	ASSERT_EQ(1, fix.b.n_incoming);
+	ASSERT_EQ(2, fix.b.n_established);
+	ASSERT_EQ(1, fix.b.n_closed);
+	ASSERT_EQ(1, fix.b.n_transfer);
+
+	ASSERT_EQ(1, fix.c.n_incoming);
+	ASSERT_EQ(1, fix.c.n_established);
+	ASSERT_EQ(0, fix.c.n_closed);
+	ASSERT_EQ(0, fix.c.n_transfer);
+
+	ASSERT_EQ(0, list_count(ua_calls(f->a.ua)));
+	ASSERT_EQ(1, list_count(ua_calls(f->b.ua)));
+	ASSERT_EQ(1, list_count(ua_calls(f->c.ua)));
+
+ out:
+	fixture_close(f);
+
+	return err;
+}
