@@ -179,8 +179,13 @@ void mock_vidcodec_unregister(void);
  */
 
 struct vidisp;
+struct vidframe;
 
-int mock_vidisp_register(struct vidisp **vidispp);
+typedef void (mock_vidisp_h)(const struct vidframe *frame, uint64_t timestamp,
+			     void *arg);
+
+int mock_vidisp_register(struct vidisp **vidispp,
+			 mock_vidisp_h *disph, void *arg);
 
 
 /* test cases */
@@ -217,6 +222,9 @@ int test_call_aulevel(void);
 int test_call_progress(void);
 int test_call_format_float(void);
 int test_call_mediaenc(void);
+int test_call_custom_headers(void);
+int test_call_tcp(void);
+int test_call_transfer(void);
 
 #ifdef USE_VIDEO
 int test_video(void);
